@@ -1,7 +1,8 @@
-import React, { FC, useState } from 'react';
+import React, { FC, useEffect, useRef, useState } from 'react';
 import { Grid } from 'react-loader-spinner';
+import { NonRunningClusterSelectOption, RunningClusterSelectOption } from './clusterSelectOption';
+import { ClusterStore } from './store';
 import { ICluster } from './types';
-
 const ChevronSVG: FC = () => (
   <svg
     height="20"
@@ -15,35 +16,11 @@ const ChevronSVG: FC = () => (
   </svg>
 );
 
-interface IOptionProps {
-  cluster: ICluster;
-  current?: boolean;
-  callback: any;
-}
-const ClusterSelectOption: FC<IOptionProps> = ({
-  cluster,
-  current = false,
-  callback,
-}: IOptionProps) => {
-  return (
-    <div
-      className={`${current ? 'current-cluster-option' : ''} bodo-cluster-dropdown-item`}
-      onClick={!current ? () => callback(cluster.uuid) : () => true}
-      key={cluster.uuid}
-    >
-      <div>
-        <span className="cluster-name">{cluster.name}</span>
-        <span className="cluster-details">Bodo {cluster.bodoVersion}</span>
-      </div>
-      {current && <button onClick={() => callback()}>Detach</button>}
-    </div>
-  );
-};
-
 interface ITriggerProps {
   cluster: ICluster | null;
   isProcessing: boolean;
 }
+
 const ClusterSelectTrigger: FC<ITriggerProps> = ({ cluster, isProcessing }: ITriggerProps) => {
   return (
     <>
@@ -77,29 +54,44 @@ interface IClusterSelectProps {
   currentClusterUUID?: string | undefined | null;
   clusters: any[];
   myOnChange: any;
+  clusterAttachloading: boolean;
+  store: ClusterStore;
 }
 
 const ClusterSelectList: FC<IClusterSelectProps> = ({
   currentClusterUUID,
   myOnChange,
   clusters,
+  clusterAttachloading,
+  store,
 }: IClusterSelectProps) => {
   const [isSelecting, setIsSelecting] = useState(false);
   const [isProcessing, setIsProcessing] = useState(false);
-
+  const ref = useRef(null);
   const clusterFromUUID = (uuid: string | undefined | null) => {
     return clusters.find((cluster) => cluster.uuid === uuid);
   };
   const renderOptions = () => {
     const currentCluster = clusterFromUUID(currentClusterUUID);
-    return clusters
+    const renderedClusters = clusters
+      .filter((cluster) => cluster.status === 'RUNNING')
+      .concat(
+        clusters.filter((cluster) => cluster.status !== 'NEW' && cluster.status !== 'RUNNING')
+      );
+    return renderedClusters
       .filter((option) => {
         if (currentCluster) {
           return currentClusterUUID !== option.uuid;
         }
         return true;
       })
-      .map((option) => <ClusterSelectOption cluster={option} callback={handleSelect} />);
+      .map((option) =>
+        option.status === 'RUNNING' ? (
+          <RunningClusterSelectOption cluster={option} callback={handleSelect} />
+        ) : (
+          <NonRunningClusterSelectOption forceUpdate={() => store.forceUpdate()} cluster={option} />
+        )
+      );
   };
 
   const handleAttachDetach = (uuid: string, detach?: boolean) => {
@@ -112,20 +104,36 @@ const ClusterSelectList: FC<IClusterSelectProps> = ({
     setIsSelecting(false);
   };
 
-  const handleSelect = (uuid: string) => handleAttachDetach(uuid, false);
-  const handleDetach = (uuid: string) => handleAttachDetach(uuid, true);
+  const handleSelect = (uuid: string) => {
+    handleAttachDetach(uuid, false);
+  };
+  const handleDetach = (uuid: string) => {
+    handleAttachDetach(uuid, true);
+  };
 
   const currentCluster = clusterFromUUID(currentClusterUUID);
   const handleToggle = () => {
     setIsSelecting(!isSelecting);
   };
+  const { current: lastCluster }: any = ref;
+  // If a cluster has been selected and it's not running then the cluster will detach, we keep track
+  // of this by comparing the ref from the last render and the current cluster.
+  if (
+    lastCluster &&
+    lastCluster.status === 'RUNNING' &&
+    (!currentCluster || currentCluster.status !== 'RUNNING')
+  ) {
+    handleDetach('0');
+  }
+  ref.current = currentCluster;
+
   return (
     <>
       <div className="bodo-cluster-select-container">
         <div onClick={handleToggle}>
           <ClusterSelectTrigger
             cluster={currentClusterUUID ? clusterFromUUID(currentClusterUUID) : null}
-            isProcessing={isProcessing}
+            isProcessing={isProcessing || clusterAttachloading}
           />
         </div>
         <div className={`bodo-cluster-select-list ${isSelecting ? 'selecting' : ''}`}>
@@ -140,7 +148,7 @@ const ClusterSelectList: FC<IClusterSelectProps> = ({
               {currentCluster ? (
                 <>
                   <div className="bodo-drop-down-details">Attached Cluster:</div>
-                  <ClusterSelectOption
+                  <RunningClusterSelectOption
                     cluster={currentCluster}
                     current={true}
                     callback={handleDetach}
