@@ -21,6 +21,7 @@ interface IState {
   clusters: ICluster[];
   currentClusterUUID: string | null;
   autoAttachState: boolean | null;
+  clusterAttachloading: boolean;
 }
 
 class ClusterSelect extends React.Component<IProps, IState> {
@@ -30,6 +31,7 @@ class ClusterSelect extends React.Component<IProps, IState> {
       clusters: props.store.clusters,
       currentClusterUUID: null,
       autoAttachState: null,
+      clusterAttachloading: false,
     };
     this._tryAutoAttach();
   }
@@ -46,12 +48,19 @@ class ClusterSelect extends React.Component<IProps, IState> {
 
   _onSessionUpdate(emitter: ISessionContext, newSession: any): void {
     console.log('Detected Session Change.');
+    const { clusters } = this.state;
     if (newSession.newValue?._kernel?._name) {
       console.debug('Session Change New Kernel Name: ', newSession.newValue._kernel._name);
       this.props.getClusterUUIDFromKernel(newSession.newValue._kernel._name).then(
         (clusterUUID) => {
-          console.debug('Found cluster for this kernel: ', clusterUUID);
-          this.setState({ currentClusterUUID: clusterUUID });
+          const clusterFromKernel = clusters.find((cluster) => cluster.uuid);
+          if (clusterFromKernel && clusterFromKernel.status === 'RUNNING') {
+            console.debug('Found cluster for this kernel: ', clusterUUID);
+            this.setState({ currentClusterUUID: clusterUUID });
+          } else {
+            console.debug('Cluster not found or not running, setting currentClusterUUID to null.');
+            this.setState({ currentClusterUUID: null });
+          }
         },
         (reason: any) => {
           console.error('Error while getting cluster UUID from kernel: ', reason);
@@ -99,48 +108,58 @@ class ClusterSelect extends React.Component<IProps, IState> {
     const { clusters, currentClusterUUID, autoAttachState } = this.state;
     let autoAttach = autoAttachState;
     let newCurrentClusterUUID = currentClusterUUID;
-
+    // Since non-running clusters are now fetched due to the addition of pausing and
+    // unpausing, clusters need to be filtered down to those that are running.
+    const runningClusters = clusters.filter((cluster) => cluster.status === 'RUNNING');
+    this.setState({ clusterAttachloading: true });
     // If autoAttach hasn't been fetched from the backend yet
     // fetch it. Since the response from the backend won't change
     // only fetch it once.
     if (autoAttachState === null) {
       autoAttach = await shouldAutoAttachHandler();
     }
-
     // If the notebook isn't attached to a cluster, there is only one cluster available
     // and is configured to have autoAttach enabled then start a kernel on the cluster
     // and set it as the currently attached cluster
-    if (!currentClusterUUID && clusters.length === 1 && autoAttach) {
-      this.props.startKernelForCluster(clusters[0].uuid).then((success) => {
+    if (!currentClusterUUID && runningClusters.length === 1 && autoAttach) {
+      this.props.startKernelForCluster(runningClusters[0].uuid).then((success) => {
         if (success) {
-          newCurrentClusterUUID = clusters[0].uuid;
+          newCurrentClusterUUID = runningClusters[0].uuid;
         }
       });
     }
-    this.setState({ currentClusterUUID: newCurrentClusterUUID, autoAttachState: autoAttach });
+
+    this.setState({
+      currentClusterUUID: newCurrentClusterUUID,
+      autoAttachState: autoAttach,
+      clusterAttachloading: false,
+    });
   }
 
   selectClusterCallback(clusterUUID: string) {
     console.debug('[onSelectCluster] clusterUUID: ', clusterUUID);
+    this.setState({ clusterAttachloading: true });
     const { startKernelForCluster } = this.props;
     if (clusterUUID === '0') {
       startKernelForCluster(); // Change to "No Kernel"
-      this.setState({ currentClusterUUID: null });
+      this.setState({ currentClusterUUID: null, clusterAttachloading: false });
     } else {
       startKernelForCluster(clusterUUID).then((success) => {
         if (success) {
-          this.setState({ currentClusterUUID: clusterUUID });
+          this.setState({ currentClusterUUID: clusterUUID, clusterAttachloading: false });
         } else {
-          this.setState({ currentClusterUUID: null });
+          this.setState({ currentClusterUUID: null, clusterAttachloading: false });
         }
       });
     }
   }
 
   render() {
-    const { clusters, currentClusterUUID } = this.state;
+    const { clusters, currentClusterUUID, clusterAttachloading } = this.state;
     return (
       <ClusterSelectList
+        store={this.props.store}
+        clusterAttachloading={clusterAttachloading}
         myOnChange={this.selectClusterCallback.bind(this)}
         clusters={clusters}
         currentClusterUUID={currentClusterUUID}
@@ -159,11 +178,15 @@ export class ToolbarExtension
   }
 
   createNew(panel: NotebookPanel, context: DocumentRegistry.IContext<INotebookModel>): IDisposable {
+    const endKernel = async () => {
+      await context.sessionContext.shutdown();
+      return true;
+    };
+
     const startKernelForCluster = async (clusterUUID?: string) => {
       if (!clusterUUID) {
         console.log('No clusterUUID. Shutting session down...');
-        await context.sessionContext.shutdown();
-        return true;
+        return endKernel();
       }
 
       console.log('Starting Kernel for clusterUUID: ', clusterUUID);
@@ -178,7 +201,7 @@ export class ToolbarExtension
       } catch (e) {
         console.error('Unable to register a remote kernel: ', e);
         showErrorMessage(
-          'Bodo Extension Errror: Unable to create a remote kernel on the cluster.',
+          'Bodo Extension Error: Unable to create a remote kernel on the cluster.',
           e
         );
         return false;
